/*
*   Definition of the categories data-structure
*   @Shahzad Ali
*/


#include "category.h"

category::category(char *n, int len, short fl) {
  valid = TRUE;
  fm = 0;
  freq = 0.0;
  g_freq = 0.0;     
  flags = 0;
  flags |= fl;
  if (len < 1){
    name = NULL;
    return;
  }
  name = new char[len];
  if (!name) {
    cout << "Out of mem" << endl;
    exit(1);
  }
  strncpy(name , n, len);
}
category::~category() {
  if (name)
    delete(name);
}

void category::set_name(char *n, int len) {
  if ( len < 1)
    return;
  name = new char[len];
  if (!name) {
    cout << "Out of mem" << endl;
    exit(1);
  }
  strncpy(name , n, len);
}

int category::increment() {
  /* Increment the frequency, return TRUE if no error */
  fm ++;
  return TRUE;
}

int category::decrement() {
  /* Decrement the frequency, return TRUE if no error */
  if (fm > 0) {
    fm --;
    return TRUE;
  }
  return FALSE;
}

/* Calculate the current frequency from the test file 
 * by using Nr, and r, if this symbol appeared
 */
void category::calc_g_freq(double Nr, double r, int saw){
  if (!Nr){
    /* Behave as if nothing happened 
     * Should probably never get here
     * in the first place */
    return;
  }
  double rate = -(log(0.5)/log(2))/r;
  double I = 1.0;
  if (saw){
    g_freq = pow(2.0, -1.0*rate)*g_freq + (I/Nr);
  }
  else 
    g_freq = pow(2.0, -1.0*rate)*g_freq;
}

/* Calculate this category's contibution to Qn */
double category::calc_Qn(double Nr){
  if (freq > 0)
    return (pow((g_freq - freq),2)/(freq));
  return -1.0;
}

/* Make a category invalid and remove all traces */
void category::invalidate() {
  valid = FALSE;
  if (name)
    delete name;
  name = NULL;
  fm = 0;
  freq = 0.0;
  flags = 0;
}
